/*
 * Copyright (c) 2004 Damir Bikmuhametov <boco@ufanet.ru>
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: common.c,v 1.1 2004/02/22 09:29:31 boco Exp $
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "common.h"

/* this procedure connects to KAV/AVE control socket and returns
 * it's filedescriptor */

int av_engine_connect(char * path) {

	int fd = -1;	/* temporary filedescriptor */
	struct sockaddr_un s;

	if (!path || !*path)
		return -1;

	memset(&s, 0 , sizeof(s));	/* clean struct */

	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
		return -1;

	s.sun_family = AF_UNIX;
	strncpy(s.sun_path, path, sizeof(s.sun_path) - 1);

	if (connect(fd, (struct sockaddr *) &s,
			       sizeof(struct sockaddr_un)) == -1) {
		/* unable to connect to AV engine */
		close(fd);
		fd = -1;
	}

	return fd;
}

/* this procedure closes connection to KAV/AVE control socket */

void av_engine_disconnect(int fd) {

	if (fd != -1)
		close(fd);
	return;
}

/* this procedure performs timed read(2) */

ssize_t timed_read(int fd, void * buf, size_t nbytes, int timeout)
{
	fd_set rdfs;
	struct timeval tv;

	tv.tv_sec = timeout;
	tv.tv_usec = 0;

	FD_ZERO(&rdfs);
	FD_SET(fd, &rdfs);
	
	if (select(fd + 1, &rdfs, NULL, NULL, timeout ? &tv : NULL) > 0)
		if (FD_ISSET(fd, &rdfs))
			return read(fd, buf, nbytes);

	errno = ETIMEDOUT;
	return -1;
}

/* this procedure performs timed write(2) */

ssize_t timed_write(int fd, const void * buf, size_t nbytes, int timeout)
{
	fd_set wfds;
	struct timeval tv;

	tv.tv_sec = timeout;
	tv.tv_usec = 0;

	FD_ZERO(&wfds);
	FD_SET(fd, &wfds);

	if (select(fd + 1, NULL, &wfds, NULL, timeout ? &tv : NULL) > 0)
		if (FD_ISSET(fd, &wfds))
			return write(fd, buf, nbytes);

	errno = ETIMEDOUT;
	return -1;
}

/* this routine frees memory occupied by "char * array[xx]" */

void free_array(void ** argv, int argc)
{
	for (; argc; argc--, argv++)
		if (*argv) free (*argv);
}

/* this routine returns pointer to line from buffer and advances buffer
   pointer up to next line */

char * getline (char ** line)
{
	char * p;

	if (!line || !*line) return NULL;
	while (**line && (**line == '\r' || **line == '\n')) (*line)++;
	p = *line;
	while (**line && **line != '\r' && **line != '\n') (*line)++;
	if (**line) {
		**line = 0;
		(*line)++;
	}
	while (**line && (**line == '\r' || **line == '\n')) (*line)++;
	return *p ? p : NULL ;
}
